package org.graphstream.graph.implementations;


import static org.junit.Assert.*;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;
import org.junit.runner.RunWith;

import java.net.URL;
import java.util.Random;

import org.databene.benerator.anno.Source;
import org.databene.feed4junit.Feeder;
import org.graphstream.graph.Edge;
import org.graphstream.graph.ElementNotFoundException;
import org.graphstream.graph.IdAlreadyInUseException;
import org.graphstream.graph.Node;
import org.junit.Test;
import org.junit.runner.RunWith;

@RunWith(Feeder.class)
public class CategoryPartitionRemoveNodeSingleGraphIt1 {

	
	//idExistance,idValue,graphNrNodes,graphStrictChecking,nodeDegree
	@Rule
	public ExpectedException exception = ExpectedException.none();
	
	@Test
	@Source("csvFiles/CategoryPartitionRemoveNodeSingleGraph.csv")
	public void test(Boolean idExistance, String idValue, String graphNrNodes, Boolean graphStrictChecking, String  nodeDegree  ) {
		
		if (idExistance && idValue.equals("correct") && graphNrNodes.equals("many") && graphStrictChecking && nodeDegree.equals("Many")) {
			test1();
		}else if(!idExistance && idValue.equals("correct") && graphNrNodes.equals("0") && graphStrictChecking && nodeDegree.equals("Many")) {
			test2();
		}else if(!idExistance && idValue.equals("correct") && graphNrNodes.equals("0") && !graphStrictChecking && nodeDegree.equals("Many")) {
			test3();
		}else if(idExistance && idValue.equals("correct") && graphNrNodes.equals("many") && !graphStrictChecking && nodeDegree.equals("Many")) {
			test4();
		}else if(!idExistance && idValue.equals("correct") && graphNrNodes.equals("many") && graphStrictChecking && nodeDegree.equals("Many")) {
			test5();
		}else if(!idExistance && idValue.equals("malformed") && graphNrNodes.equals("0") && graphStrictChecking && nodeDegree.equals("Many")) {
			test6();
		}else if(!idExistance && idValue.equals("") && graphNrNodes.equals("0") && graphStrictChecking && nodeDegree.equals("Many")) {
			test7();
		}else if(idExistance && idValue.equals("null") && graphNrNodes.equals("many") && graphStrictChecking && nodeDegree.equals("Many")) {
			test8();
		}else if(!idExistance && idValue.equals("correct") && graphNrNodes.equals("0") && graphStrictChecking && nodeDegree.equals("0")) {
			test9();
		}
	}
	
	// to be review
	  public void test1() {
//		    true,correct,many,true,Many
			SingleGraph single = new SingleGraph("single"); // strict checking will be set auto to true by the constructor.
			for (int i=0; i<10; i++) {
				single.addNode("node"+i);
			}
			for (int i = 0; i < 4; i++) {
				single.addEdge("Edge_"+i, "node1", "node"+(i+1));
			}
//			exception.expect(ElementNotFoundException.class);
			single.removeNode("node1");	
			assertTrue(single.getNodeCount() == 9);
		
		}
	
	
	  public void test2() {
//		  	false,correct,0,true,Many
			SingleGraph single = new SingleGraph("single"); // strict checking will be set auto to true by the constructor.
			for (int i=0; i<10; i++) {
				single.addNode("node"+i);
			}
			for (int i = 0; i < 4; i++) {
				single.addEdge("Edge_"+i, "node1", "node"+(i+1));
			}
			exception.expect(ElementNotFoundException.class);
			single.removeNode("notExistedNode");
			fail();
		}
	  

	  
	  public void test3() {
//		  	false,correct,0,false,Many
			SingleGraph single = new SingleGraph("single",false,false);
			for (int i=0; i<10; i++) {
				single.addNode("node"+i);
			}
			for (int i = 0; i < 4; i++) {
				single.addEdge("Edge_"+i, "node1", "node"+(i+1));
			}
			Node n1 = single.removeNode("notExistedNode");	
			assertEquals(n1, null);
		}
	  
	  public void test4() {
//		    true,correct,many,false,Many
			SingleGraph single = new SingleGraph("single",false,false); 
			for (int i=0; i<10; i++) {
				single.addNode("node"+i);
			}
			for (int i = 0; i < 4; i++) {
				single.addEdge("Edge_"+i, "node1", "node"+(i+1));
			}
			single.removeNode("node1");	
			assertTrue(single.getNodeCount() == 9);
		}
	  
	  public void test5() {
//		    false,correct,many,true,Many
			SingleGraph single = new SingleGraph("single"); // strict checking will be set auto to true by the constructor.
			String idValue = "NotExistedNode";
			for (int i=0; i<10; i++) {
				single.addNode("node"+i);
			}
			for (int i = 0; i < 4; i++) {
				single.addEdge("Edge_"+i, "node1", "node"+(i+1));
			}
			exception.expect(ElementNotFoundException.class);
			single.removeNode(idValue);	
		}
	  
	  public void test6() {
//		  	false,malformed,0,true,Many
			SingleGraph single = new SingleGraph("single"); // strict checking will be set auto to true by the constructor.
			String idValue = "nodeNew";

			exception.expect(ElementNotFoundException.class);
			single.removeNode(idValue);	
		}
	  
	  
	  public void test7() {
//		  	false,'',0,true,Many
			SingleGraph single = new SingleGraph("single"); // strict checking will be set auto to true by the constructor.
			String idValue = "nodeNew";
			
			exception.expect(ElementNotFoundException.class);
			single.removeNode(idValue);	
		}
	  
	  public void test8() {
//		    true,null,many,true,Many
			SingleGraph single = new SingleGraph("single"); // strict checking will be set auto to true by the constructor.
			String idValue = "nodeNew";
			for (int i=0; i<10; i++) {
				single.addNode("node"+i);
			}
			for (int i = 0; i < 4; i++) {
				single.addEdge("Edge_"+i, "node1", "node"+(i+1));
			}
			exception.expect(ElementNotFoundException.class);
			single.removeNode("nodeNotInGraph");	
		}
	  public void test9() {
//		    false,correct,0,true,0
			SingleGraph single = new SingleGraph("single"); // strict checking will be set auto to true by the constructor.
			String idValue = "nodeNew";
			
			exception.expect(ElementNotFoundException.class);
			single.removeNode(idValue);	
		}

	

}
